@@ -128,10 +128,11 @@ class EventsFrame(object):
     default_end =  ['EMP', 'END', 'TO']
     default_geom = ['geometry']
 
-    def __init__(self, df, keys=None, beg=None, end=None, geom=None, 
-                 route=None, closed='left_mod', sort=False, **kwargs):
+    def __init__(
+        self, df, keys=None, beg=None, end=None, geom=None, route=None, 
+        closed='left_mod', sort=False, **kwargs):
         # Log input values
-        super(EventsFrame, self).__init__(**kwargs)
+        super(EventsFrame, self).__init__()
         self._df = df
         self.keys = keys
         self.beg = beg
@@ -171,8 +172,8 @@ class EventsFrame(object):
             # Sort dataframe
             self._df = df
             if self._sort:
-                self._df = df.sort_values(by=self.keys+[self.beg,self.end],
-                                          ascending=True)
+                self._df = self._sort_df(df)
+
             # Define the key groups
             if self.num_keys > 0:
                 self._groups = self._df.groupby(by=self.keys)
@@ -186,6 +187,29 @@ class EventsFrame(object):
             raise TypeError("Input dataframe must be pandas DataFrame class "
                 "instance.")
 
+    def _sort_df(self, df):
+        """
+        Sort the given dataframe by the collection's keys and begin/end 
+        columns, returning the sorted dataframe.
+        """
+        return df.sort_values(
+            by=self.keys + [self.beg, self.end], ascending=True)
+
+    def df_exportable(self):
+        """
+        Return a dataframe which is optimized for exporting.
+        """
+        # Create a copy of the events dataframe
+        df = self.df.copy()
+        # Convert route data to wkt
+        try:
+            df[self.route] = \
+                df[self.route].apply(lambda x: x.wkt)
+        except:
+            # Remove route column
+            df = df.drop(columns=[self.route], errors='ignore')
+        return df
+
     @property
     def keys(self):
         """
@@ -375,6 +399,63 @@ class EventsFrame(object):
         self._route_loc = self.columns.index(route) \
                 if not route is None else None
 
+    def parse_routes(self, col=None, inplace=False, errors='raise'):
+        """
+        Parse MLSRoutes data in the provided column, which contains either 
+        MLSRoute objects, WKT data for MULTILINESTRINGs or LINESTRINGs with 
+        M-values, or a mixture of both.
+
+        Parameters
+        ----------
+        col : label, optional
+            A valid column label within the events dataframe which contains the 
+            target MLSRoute data. If not provided, will attempt to retrieve a 
+            previously assigned column label from the self.route property.
+        inplace : boolean, default False
+            Whether to perform the operation in place. If False, will return a 
+            modified copy of the events object.
+        errors : {'raise','ignore'}
+            How to address errors which arise when coercing MLSRoute data 
+            during processing. If ignored, errors will result in null values 
+            in the events dataframe where errors occurred.
+        """
+        # Check column
+        if col is None:
+            try:
+                col = self._route
+            except:
+                raise ValueError("No route column label provided.")
+        # Coerce data
+        def _to_routes(x):
+            if isinstance(x, MLSRoute):
+                return x
+            elif isinstance(x, str):
+                try:
+                    return MLSRoute.from_wkt(x)
+                except Exception as e:
+                    if errors=='raise':
+                        raise e
+                    else:
+                        return
+            else:
+                if errors=='raise':
+                    raise TypeError(
+                        "Route data must be MLSRoute object or WKT valid "
+                        "string.")
+                else:
+                    return
+        routes = self.df[col].apply(_to_routes)
+        # Apply update
+        if inplace:
+            self.df[col] = routes
+            self.route = col
+            return
+        else:
+            ec = self.copy(deep=True)
+            ec.df[col] = routes
+            ec.route = col
+            return ec
+
     @property
     def closed(self):
         """
@@ -515,6 +596,9 @@ class EventsFrame(object):
         reorder=True, merge_lines=True):
         """
         Dissolve the events dataframe on a selection of event attributes.
+
+        Note: Data will be sorted by keys and begin/end columns prior to 
+        performing the dissolve.
         
         Note: Missing data in selected attribute fields may cause problems with 
         dissolving; please use df.fillna(...) or df.dropna(...) to avoid this 
@@ -549,7 +633,8 @@ class EventsFrame(object):
             Whether to drop records with empty values in the attribute fields. 
             This parameter is passed to the df.groupby call.
         fillna : optional
-            A value used to fill instances of np.nan in the target dataframe.
+            A value or dictionary used to fill instances of np.nan in the 
+            target dataframe. Consistent with the DataFrame.fillna() method.
         reorder : bool, default True
             Whether to reorder the resulting dataframe columns to match the 
             order of the collection's events dataframe.
@@ -559,8 +644,8 @@ class EventsFrame(object):
             applicable when agg_geometry=True.
         """
         # Validate inputs
-        # - Create dummy dataframe
-        df = self.df.copy()
+        # - Create, sort dummy dataframe
+        df = self._sort_df(self.df.copy())
         col_order = list(df.columns)
         df['__DUMMY__'] = True # Dummy data guarantees >0 groupby keys
         # - Dissolve attributes
@@ -662,9 +747,15 @@ class EventsFrame(object):
             # Get aggregation data
             lin_ranges = np.split(np.stack([begs_i, ends_i]), splitter, axis=1)
             if not agg_groups is None:
-                agg_data = agg_groups.loc[index, :].to_list()
-                agg_ranges = [[agg[i:j] for agg in agg_data] for i,j in \
-                            zip([None]+splitter, splitter+[None])]
+                try:
+                    agg_data = agg_groups.loc[index, :].to_list()
+                    agg_ranges = [[agg[i:j] for agg in agg_data] for i,j in \
+                                zip([None]+splitter, splitter+[None])]
+                except KeyError:
+                    raise KeyError(
+                        f"Unable to retrieve data group with index {index}. "
+                        "This may be due to nan data in one or more of the "
+                        "dissolving attributes.")
             else:
                 agg_ranges = iter(list, 1)
             
@@ -706,8 +797,109 @@ class EventsFrame(object):
             closed=self.closed)
         return ec
 
-    def project(self, other, buffer=100, choose='min', loc_label='LOC', 
-            dist_label='DISTANCE'):
+    def project(self, other, buffer=100, nearest=True, loc_label='LOC', 
+            dist_label='DISTANCE', **kwargs):
+        """
+        Project an input geodataframe onto the events dataframe, producing 
+        linearly referenced point locations relative to events for all input 
+        geometries within a buffered search area.
+
+        Parameters
+        ----------
+        other : gpd.GeoDataFrame
+            Geodataframe containing geometry which will be projected onto the 
+            events dataframe.
+        buffer : float, default 100
+            The max distance to search for input geometries to project against 
+            the events' geometries. Measured in terms of the geometries' 
+            coordinate reference system.
+        nearest : bool, default True
+            Whether to choose only the nearest match within the defined buffer. 
+            If False, all matches will be returned.
+        loc_label, dist_label : label
+            Labels to be used for created columns for projected locations on 
+            target events groups and nearest point distances between target 
+            geometries and events geometries.
+        **kwargs
+            Keyword arguments to be passed to the EventsFrame constructor 
+            upon completion of the projection.
+        """
+        # Validate input geodataframe
+        if not isinstance(other, gpd.GeoDataFrame):
+            raise TypeError("Other object must be gpd.GeoDataFrame instance.")
+        other = other.copy()
+
+        # Check for invalid column names
+        if self.route in other.columns:
+            raise ValueError(f"Invalid column name '{self.route}' found in "
+                "target geodataframe.")
+
+        # Ensure that geometries and routes are available
+        if self.geom is None:
+            raise ValueError("No geometry found in events dataframe. If "
+                "valid shapely geometries are available in the dataframe, "
+                f"set this with the {self.__class__.__name__}'s geom "
+                "property.")
+        elif self.route is None:
+            raise ValueError("No routes found in events dataframe. If valid "
+                "shapely geometries are available in the dataframe, create "
+                "routes by calling the build_routes() method on the "
+                f"{self.__class__.__name__} class instance.")
+        
+        # Join the other geodataframe to this one
+        select_cols = self.keys + [self.route, self.geom]
+        try:
+            if nearest:
+                joined = other.sjoin_nearest(
+                    self.df[select_cols],
+                    max_distance=buffer,
+                    how='left'
+                )
+            else:
+                warnings.warn(
+                    "Performance when nearest=False is currently limited and "
+                    "will be improved in future versions.")
+                joined = join_nearby(
+                    other, 
+                    self.df[select_cols], 
+                    buffer=buffer, 
+                    choose='all',
+                    dist_label=dist_label
+                )
+        except AttributeError:
+            # Optional dependency warning for improved performance
+            warnings.warn(
+                "Performance will be reduced for this operation when using "
+                "the current geopandas version. Upgrade to geopandas>=v0.10.0 "
+                "for improved performance.")
+            joined = join_nearby(
+                other, 
+                self.df[select_cols], 
+                buffer=buffer, 
+                choose='min' if nearest else 'all',
+                dist_label=dist_label
+            )
+
+        # Project input geometries onto event geometries
+        def _project(r):
+            try:
+                return r[self.route].project(r.geometry)
+            except AttributeError:
+                return
+        locs = joined.apply(_project, axis=1)
+        joined[loc_label] = locs
+
+        # Prepare and return data
+        return self.__class__(
+            joined.drop(columns=[self.route]),
+            keys=self.keys,
+            beg=loc_label,
+            closed=self.closed,
+            **kwargs
+        )
+
+    def project_old(self, other, buffer=100, choose='min', loc_label='LOC', 
+            dist_label='DISTANCE', **kwargs):
         """
         Project an input geodataframe onto the events dataframe, producing 
         linearly referenced point locations relative to events for all input 
@@ -737,6 +929,9 @@ class EventsFrame(object):
             Labels to be used for created columns for projected locations on 
             target events groups and nearest point distances between target 
             geometries and events geometries.
+        **kwargs
+            Keyword arguments to be passed to the EventsFrame constructor 
+            upon completion of the projection.
         """
         # Validate input geodataframe
         if not isinstance(other, gpd.GeoDataFrame):
@@ -780,7 +975,13 @@ class EventsFrame(object):
         joined[loc_label] = locs
 
         # Prepare and return data
-        return joined.drop(columns=[self.route])
+        return self.__class__(
+            joined.drop(columns=[self.route]),
+            keys=self.keys,
+            beg=loc_label,
+            closed=self.closed,
+            **kwargs
+        )
 
     def to_windows(self, dissolve=False, **kwargs):
         """
@@ -1416,6 +1617,36 @@ class EventsCollection(EventsFrame):
                 # Single group
                 return self.get_group(keys, empty=False)
 
+    def from_similar(self, df, **kwargs):
+        """
+        Create an EventsCollection from the input dataframe, assuming the same 
+        column labels and closed parameter as the calling collection. 
+        Additional constructor keyword arguments can be passed through 
+        **kwargs.
+
+        Parameters
+        ----------
+        df : pd.DataFrame
+            Pandas dataframe which contains linear or point events data, 
+            formatted with standard labels. If multiple keys are detected, they 
+            will be assigned in the order in which they appear within the 
+            target dataframe. Only one of each begin and end option may be 
+            used. The geometry label is optional.
+        **kwargs
+            Additional keyword arguments to be passed to the EventsCollection 
+            constructor.
+        """
+        # Build the events collection
+        kwargs = {**dict(
+            keys=self.keys,
+            beg=self.beg,
+            end=self.end,
+            geom=self.geom,
+            closed=self.closed,
+        ), **kwargs}
+        ec = self.__class__(df, **kwargs)
+        return ec
+
     @classmethod
     def from_standard(cls, df, require_end=False, **kwargs):
         """
@@ -1433,6 +1664,9 @@ class EventsCollection(EventsFrame):
         end : 'EMP', 'END', 'TO'
         geom : 'geometry'
 
+        Additional constructor keyword arguments can be passed through 
+        **kwargs.
+
         Parameters
         ----------
         df : pd.DataFrame
@@ -1445,6 +1679,9 @@ class EventsCollection(EventsFrame):
             Whether to raise an error if no valid unique end column label is 
             found. If False, no end label will be used when generating the 
             collection.
+        **kwargs
+            Additional keyword arguments to be passed to the EventsCollection 
+            constructor.
         """
         # Check for standard label assignments
         keys, beg, end, geom = [], None, None, None
@@ -1590,7 +1827,7 @@ class EventsCollection(EventsFrame):
         return em
 
     def project_parallel(self, other, samples=3, buffer=100, match='all', 
-            choose=1, sort_locs=True):
+            choose=1, sort_locs=True, **kwargs):
         """
         Project an input geodataframe of linear geometries onto parallel events 
         in the events dataframe, producing linearly referenced locations for all 
@@ -1618,11 +1855,21 @@ class EventsCollection(EventsFrame):
         sort_locs : bool, default True
             Whether begin and end location values should be sorted, ensuring 
             that all events are increasing and monotonic.
+        **kwargs
+            Keyword arguments to be passed to the EventsCollection constructor 
+            upon completion of the projection.
         """
         # Create projector
         pp = ParallelProjector(self, other, samples=samples, buffer=buffer)
-        # Perform match and return results
-        return pp.match(match=match, choose=choose, sort_locs=sort_locs)
+        # Perform match and return results in new events collection
+        return EventsCollection(
+            pp.match(match=match, choose=choose, sort_locs=sort_locs),
+            keys=self.keys,
+            beg=self.beg,
+            end=self.end,
+            closed=self.closed,
+            **kwargs
+        )
     
     def get_group(self, keys, empty=True, **kwargs) -> EventsGroup:
         """
@@ -1806,6 +2053,52 @@ def get_mode(arr):
     return unique[np.argmax(counts)]
 
 
+####################
+# COMMON FUNCTIONS #
+####################
+
+def check_compatibility(objs, errors='raise', **kwargs):
+    """
+    Check if the input list of EventsCollections are all compatible for 
+    merging, unifying, or similar relational processes. Errors will be raised 
+    if objects are not found to be compatible with information about why they 
+    are not compatible. If requested, errors can be ignored, returning False 
+    instead. If all objects are compatible, the function will return True.
+
+    Parameters
+    ----------
+    objs : list-like of EventsCollections
+        List of EventsCollection objects to be tested against each other.
+    errors : {'raise','ignore'}
+        How to respond to errors when they arise.
+    """
+    # Ensure minimum objects provided
+    try:
+        assert len(objs) > 0
+    except AssertionError:
+        raise ValueError("Must provide at least one object for testing.")
+    try:
+        # Ensure type
+        try:
+            assert all(isinstance(obj, EventsCollection) for obj in objs)
+        except AssertionError:
+            raise TypeError("All input objects must be EventsCollections.")
+        # Ensure matching keys
+        try:
+            num_keys = objs[0].num_keys
+            for obj in objs[1:]:
+                assert obj.num_keys == num_keys
+        except AssertionError:
+            raise ValueError(
+                "All input objects must have the same number of keys.")
+    except Exception as e:
+        if errors == 'raise':
+            raise e
+        else:
+            return False
+    return True
+
+
 #####################
 # LATE DEPENDENCIES #
 #####################
